function SetButtonActive(self, button, active)
	local attr = self:GetAttribute()
	local activeTextColor = attr.ActiveTextColor
	local normalTextColor = attr.NormalTextColor
	
	if active then
		button:SetNormalBkgTexture(attr.ActiveBtnBkgNormal)
		button:SetHoverBkgTexture(attr.ActiveBtnBkgHover)
		button:SetDownBkgTexture(attr.ActiveBtnBkgDown)
		button:SetDisableBkgTexture(attr.ActiveBtnBkgDisable)
		--button:SetTextColorID("system.white")   
		button:SetTextColorID(activeTextColor)   
	else
		button:SetNormalBkgTexture(attr.BtnBkgNormal)
		button:SetHoverBkgTexture(attr.BtnBkgHover)
		button:SetDownBkgTexture(attr.BtnBkgDown)
		button:SetDisableBkgTexture(attr.BtnBkgDisable)
		--button:SetTextColorID("system.gray") 
		button:SetTextColorID(normalTextColor) 
	end
end

function SetText(self,id,strText)
	local attr = self:GetAttribute()
	if not attr or not id then
		return
	end
	
	local obj = self:GetControlObject(id)
	if not obj then
		return
	end
	
	obj:SetText(strText)
end

--	active:bool 是否激活
function AddTabItem(self, id, text, icon, active)
	local objExist = self:GetControlObject(id)
	if objExist ~= nil then
		return
	end
	local objFactory = XLGetObject("Xunlei.UIEngine.ObjectFactory")

	local attr = self:GetAttribute()
	local direction = attr.Direction  
	local bkgObj = self:GetControlObject("bkg")
	local btn = objFactory:CreateUIObject(id, "Thunder.ImageTextButton")
	
	local btnAttr = btn:GetAttribute()
	btnAttr.IconLeftPos = attr.IconLeftPos
	btnAttr.IconTopPos = attr.IconTopPos
	btnAttr.TextLeftPos = attr.TextLeftPos
	btnAttr.TextTopPos = attr.TextTopPos
	btnAttr.IconSize = attr.IconSize
	btnAttr.IconBitmapID = icon
	btnAttr.Text = text

	btnAttr.TextFontID = attr.TextFontID
	btnAttr.TextColorID = attr.TextColorID
	
	local pos = 0
	local btnCount = bkgObj:GetChildCount()
	if btnCount > 0 then
		local lastBtn = bkgObj:GetChildByIndex(btnCount - 1)
		local left, top, right, bottom = lastBtn:GetObjPos()
		if direction == 'H' then
			pos = right + attr.ButtonInternalSpace
		else
			pos = bottom + attr.ButtonInternalSpace
		end
	end

	local function OnButtonClick(btn)
		local id = btn:GetID()
		if id == attr.CurrentActiveTab then
			return
		end
		self:SetActiveTab(id)
	end
	btn:AttachListener("OnButtonClick", true, OnButtonClick)
	bkgObj:AddChild(btn)

	--XLMessageBox( attr.ButtonWidth) 
	if direction == 'H' then
		btn:SetObjPos(pos, 0, pos + attr.ButtonWidth, 0 + attr.ButtonHeight)
	else
		btn:SetObjPos(0, pos, attr.ButtonWidth, pos + attr.ButtonHeight)
	end
	
	if active == true or
		(active == nil and attr.CurrentActiveTab == nil) then
		self:SetActiveTab(id, false)
	else
		SetButtonActive(self, btn, false)
	end
end



function RemoveTabItem(self, remove_id, active_id)
	local btn = self:GetControlObject(remove_id)
	if btn == nil then
		return
	end
	local bkgObj = self:GetControlObject("bkg")
	local attr = self:GetAttribute()
	local pos = 0
	local count = bkgObj:GetChildCount()
	for i = 0, count - 1 do
		local child = bkgObj:GetChildByIndex(i)
		local childID = child:GetID()
		if childID ~= remove_id then
			local childLeft, childTop, childRight, childBottom = child:GetObjPos()
			child:SetObjPos(pos, childTop, pos + attr.ButtonWidth, childBottom)
			pos = pos + attr.ButtonWidth + attr.ButtonInternalSpace
		end
	end
	bkgObj:RemoveChild(btn)
end

function SetActiveTab(self, tabID, fireEvent)
	local attr = self:GetAttribute()
	if attr.CurrentActiveTab == tabID then
		return
	end

	local btn = self:GetControlObject(tabID)
	if btn == nil then
		return
	end

	SetButtonActive(self, btn, true)
	
	local pre_active = attr.CurrentActiveTab
	if pre_active ~= nil then
		local btn = self:GetControlObject(pre_active)
		if btn == nil then
			return
		end
		SetButtonActive(self, btn, false)
	end
	
	attr.CurrentActiveTab = tabID
	if fireEvent ~= false then
		self:FireExtEvent("OnActiveTabChanged", tabID, pre_active)
	end
end

function OnInitTabCtrlBkg(self)
	local attr = self:GetAttribute()
	local bkg = self:GetControlObject("bkg")
	bkg:SetTextureID(attr.BorderTexture)
end

pageIndex = 0  
function AddPage(self,classID,instanceID)
	local attr = self:GetAttribute()
	if attr.Pages == nil then
		attr.Pages = {}
	end
	for k,v in pairs(attr.Pages) do
		if v.id == instanceID then
			return false
		end
	end 
    
	attr.Pages[#(attr.Pages)+1] = {class=classID,id=instanceID,obj=nil,pageIndex=pageIndex}
	pageIndex = pageIndex + 1   
	return true
end

isFirstInto = true  
function ActivePage(self,instanceID)
	local attr = self:GetAttribute()
	if attr.Pages == nil then
		return false
	end
	
	for k,v in pairs(attr.Pages) do
		if v.id == instanceID then
			if attr.selectPage == v then
				return true
			end
			
			if v.obj == nil then
				-- XLMessageBox(v.id.." "..v.class)  
			end
			-- 不销毁模式　　
			if v.obj == nil then
				local xarManager = XLGetObject("Xunlei.UIEngine.XARManager")
				local xarFactory = xarManager:GetXARFactory()
				local obj = xarFactory:CreateUIObject(v.id,v.class)
				if obj == nil then
					return false;
				end
				
				-- 没有转场动画
				if attr.Transition == '0' then
					obj:SetObjPos("0","0","father.width","father.height")
				end 
				
				-- 左右滑动转场
				if attr.Transition == '1' then
					if attr.selectPage ~= nil then
						if v.pageIndex > attr.selectPage.pageIndex then -- 从右边进来 
							--XLMessageBox("from right")
							obj:SetObjPos("father.width","0","2*father.width","father.height")  
						else
							--XLMessageBox("from left")
							obj:SetObjPos("-father.width","0","0","father.height") 
						end
					else
						obj:SetObjPos("0","0","father.width","father.height") 
					end
				end
				v.obj = obj
				self:GetControlObject("bkg"):AddChild(v.obj)
			end 
			
			--没有转场动画
			if attr.Transition == '0' then
				if attr.selectPage ~= nil then
					local selectObj = attr.selectPage.obj 
					selectObj:SetVisible(false)  
					selectObj:SetChildrenVisible(false)
				end
				v.obj:SetVisible(true)  
				v.obj:SetChildrenVisible(true) 
				attr.selectPage = v 
				return true
			end 
			
			-- 左右滑动转场
			if attr.Transition == '1' then
				-- 从模版创建动画实例 　
				local templateManager = XLGetObject("Xunlei.UIEngine.TemplateManager")	
				local posAniTemplate = templateManager:GetTemplate("thunder.tabctrl.pageani","AnimationTemplate")
				local posAniInstance1 = posAniTemplate:CreateInstance() 
				local posAniInstance2 = posAniTemplate:CreateInstance() 
				--local ownerControl = self:GetOwnerControl()	
				-- 获取control所属treeobject　直接self:getowner是不行的 　 
				local treeObject = self:GetOwner()
				treeObject:AddAnimation(posAniInstance1) 
				treeObject:AddAnimation(posAniInstance2) 
		
				if attr.selectPage ~= nil then
					--self:GetControlObject("bkg"):RemoveChild(attr.selectPage.obj)
					--attr.selectPage.obj = nil
					local selectObj = attr.selectPage.obj  
					local sleft,stop,sright,sbottom = selectObj:GetObjPos()
					--XLMessageBox(sleft.." "..stop.." "..sright.." "..sbottom)
					local swidth = sright - sleft  
					posAniInstance1:BindLayoutObj(selectObj)
					if v.pageIndex > attr.selectPage.pageIndex then -- 向左移出屏幕  
						--XLMessageBox("to left")
						posAniInstance1:SetKeyFrameRect(
							sleft,stop,sright,sbottom,
							sleft - swidth,stop,sright - swidth,sbottom
						)
					else  --向右移出屏幕 　
						posAniInstance1:SetKeyFrameRect(
							sleft,stop,sright,sbottom,
							sleft + swidth,stop,sright + swidth,sbottom
						)
					end
					local function onAniFinish1(self,old,new)
						if new == 4 then
							selectObj:SetVisible(false)  
							selectObj:SetChildrenVisible(false)  
						end
					end   	 
							
					posAniInstance1:AttachListener(true ,onAniFinish1)
					posAniInstance1:Resume() 
					--attr.selectPage.obj.visible = 0     
					--XLMessageBox(attr.selectPage.id.." "..attr.selectPage.class)  
				end
				--attr.Pages[k] = v   
			
				v.obj:SetVisible(true)  
				v.obj:SetChildrenVisible(true)  
				--XLMessageBox(v.pageIndex)
				if isFirstInto then   -- 第一次不显示动画
					isFirstInto = false
				else
					posAniInstance2:BindLayoutObj(v.obj)
					local vleft,vtop,vright,vbottom = v.obj:GetObjPos()
					--XLMessageBox(vleft.." "..vtop.." "..vright.." "..vbottom)
					local vwidth = vright - vleft
					--XLMessageBox(v.pageIndex .. " " .. attr.selectPage.pageIndex)  
					if v.pageIndex > attr.selectPage.pageIndex then -- 从右边进来   
						--XLMessageBox("from right")
						--XLMessageBox(vleft - vwidth.." "..vtop.." "..vright - vwidth.." "..vbottom)
						posAniInstance2:SetKeyFrameRect(
							vleft,vtop,vright,vbottom,
							vleft - vwidth,vtop,vright - vwidth,vbottom
						)
					else  --从左边进来 
						posAniInstance2:SetKeyFrameRect(
							vleft,vtop,vright,vbottom,
							vleft + vwidth,vtop,vright + vwidth,vbottom
						)
					end
					local function onAniFinish2(self,old,new)
						if new == 4 then
							v.obj:SetObjPos("0","0","father.width","father.height")  
						end
					end   	 
				
					posAniInstance2:AttachListener(true ,onAniFinish2)   
					posAniInstance2:Resume() 
				end 
				
				attr.selectPage = v 
			end
			return true
		end
	end
	
	return false
end